<%@page import="
       org.w3c.dom.Element,
       org.w3c.dom.NodeList,
       org.w3c.dom.Document,
       javax.xml.parsers.DocumentBuilder,
       javax.xml.parsers.DocumentBuilderFactory,
       java.util.HashMap"
%><%!
/**
 * FeedServer is a class that is designed to provide a nice JavaScript output 
 * from one or more RSS feeds. The object maintains a list of valid RSS sources
 * and retrieves them based on a parameter that is retrieved from the HTTP request.
 * 
 * The end user will call this page using a scriptlet within their HTML document. 
 * 
 * Example: <script src="http://somedomain.com/serveFeed.jsp?hed=4&feed=1&desc=yes"></script>
 * 
 * Allowed URL parameters:
 * 
 * hed=4    this indicates the number of headlines to display, in this case 4
 *          the default is 10
 * feed=1   this indicates which feed to use. the feed is defined in the setupFeeds() method
 *          the default is feed 1
 * desc=yes this indicates whether to display descriptions in addition to headlines
 *          the default is no
 * 
 * @author      Brian Schaffner
 * @version     %I%, %G%
 * @since       1.0
 */

public class FeedServer {
    String feedType;
    String headlines;
    String feedURL;
    String showDesc;
    int nheads;
    StringBuffer output;	
    HashMap feeds;

    /**
    * Fills out the feeds hashmap with the available feed types and URLs. If you wanted	
    * to add additional feeds to process, this is where you would do it. Minimally, you should
    * always at least have a feed type of "1". If not, it will affect the default feed provided
    * in the getFeed() method.
    *
    */
    private void setupFeeds() {
        if (feeds != null) {
            feeds.put("1", "http://builder.com.com/5150-22-0.xml");
            feeds.put("2", "http://techrepublic.com.com/5150-22-0.xml");
        }
    }

    /**
    * Returns a String that contains the URL for a feed based on the feed type currently
    * defined for the object.
    * 
    * @return      a string containing the URL to the RSS feed
    */
    private String getFeed() {
        String retval = "";
        retval = (String) feeds.get(feedType);
        if (retval == null) {
            // when the feed type is not provided, use a default of "1"
            retval = (String) feeds.get("1");
        }
        return retval;
    }

    /**
    * Constructor for the FeedServer. Calls the class initializer
    *
    */
    public FeedServer() {
        init();
    }

    /**
    * Initializer for the FeedServer. Sets default values for the object's members.
    *
    */
    private void init() {
        output = new StringBuffer();
        feeds = new HashMap();
        setupFeeds();
        feedType = "1";
        headlines = "10";
        feedURL = getFeed();
        nheads = 10;
        showDesc = "no";		
    }	

    /**
    * Returns a String that contains the necessary JavaScript to render the feed
    * in the user's browser. 
    * The request argument must contain the associated HttpServletRequest object. 
    * <p>
    * @param  request	the actual HttpServletRequest for this feed request
    * @return      a string containing the JavaScript for rendering
    */
    public String generateFeed(HttpServletRequest request) {
        Document doc;
        getFormParameters(request);
        doc = getXMLDocForContentFeed();
        return createOutputFromXMLDoc(doc);
    }

    /**
    * Retrieves the object's members by interrogating the parameters provided in the request.
    *
    * @param	request	the actual HttpServletRequest for this feed request
    */
    private void getFormParameters(HttpServletRequest request) {
        try {
            // this provides the params and validation
            // null input params are assigned back to their defaults (defined in the constructor)
            feedType = (request.getParameter("feed") == null ? feedType : request.getParameter("feed"));
            headlines = (request.getParameter("hed") == null ? headlines : request.getParameter("hed"));
            showDesc = (request.getParameter("desc") == null ? showDesc : request.getParameter("desc"));
            nheads = new Integer(headlines).intValue();
            feedURL = getFeed();
        } catch (Exception e) {
            // if there's a problem, we'll re-init the defaults
            init();			
        }
    }	

    /**
    * Returns a Document containing the parsed XML content for the associated feed.
    *
    * @return  a Document with the parsed feed data
    */
    private Document getXMLDocForContentFeed() {
        Document retval = null;
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();				
            DocumentBuilder parser = factory.newDocumentBuilder();
            retval = parser.parse(feedURL); 
        } catch (Exception e) {
            // this state is currently undefined
            output.append("document.write('Failure retrieving RSS feed')\n");		
        }
        return retval;
    }

    /**
    * Returns a String that has been transformed to have all single quotes (apostrophes) 
    * prepended with a backslash escape character. This is necessary for making the 
    * JavaScript output work properly.
    *
    * @param    s    the String to be escaped
    * @return   a    transformed String
    */
    private String escapeStr(String s) {  
        char[] c = s.toCharArray();  
        StringBuffer sb = new StringBuffer();  
        for (int i = 0; i < c.length; i++) {    
            if (c[i] == '\'') sb.append("\\");
            sb.append(c[i]);
        }
        return sb.toString();		
    }

    /**
    * Returns a String that contains the formatted JavaScript output based on the 
    * RSS feed contained in the provided Document.
    *
    * @param    doc  the XML document containing the RSS feed data
    * @return   a    String containing the JavaScript output
    */	
    private String createOutputFromXMLDoc(Document doc) {
        Element root, image;
        NodeList nlist, images;
        String title, link, description, language, copyright, pubdate, lastbuilddate, category, generator, docs, ttl;
        String imtitle, imurl, imlink, imwidth, imheight;
        String itemtitle, itemlink, itemdescription, itemcategory, itemguid, itempubdate;
        if (doc != null) {
            // might like to put these items into 
            // their own RSS class(es)
            root = doc.getDocumentElement();
            title = getElementValue(root, "title");
            link = getElementValue(root, "link");
            description = getElementValue(root, "description");
            language = getElementValue(root, "language");
            copyright = getElementValue(root, "copyright");
            pubdate = getElementValue(root, "pubDate");
            lastbuilddate = getElementValue(root, "lastBuildDate");
            category = getElementValue(root, "category");
            generator = getElementValue(root, "generator");
            docs = getElementValue(root, "docs");
            ttl = getElementValue(root, "ttl");
	
            images = root.getElementsByTagName("image");
            image = (Element) images.item(0);
            imtitle = getElementValue(image, "title");
            imurl = getElementValue(image, "url");
            imlink = getElementValue(image, "link");
            imwidth = getElementValue(image, "width");
            imheight = getElementValue(image, "height");
	
            output.append("document.write('<link rel=\"stylesheet\" type=\"text/css\" href=\"http://techrepublic.com.com/css/builderstyle.css\" />');\n");	
            output.append("document.write('<table border=\"0\">')\n");		
            output.append("document.write('<tr><td><a href=\"" + imlink + "\"><img src=\"" + imurl + "\" border=\"0\" width=\"" + imwidth + "\" height=\"" + imheight + "\"></a><br></td></tr>')\n");		
            output.append("document.write('<tr><td><img src=\"http://builder.com.com/i/tr/pentagon.gif\" width=\"15\" height=\"15\"><a href=\"" + link + "\"><font class=\"navSectionText\">" + title + "</font></a></td></tr>')\n");

            nlist = root.getElementsByTagName("item");
            for (int i = 0; i < nlist.getLength() && i < nheads; i++) {
                Element e = (Element) nlist.item(i);
                itemtitle = getElementValue(e, "title");
                itemlink = getElementValue(e, "link");
                itemdescription = getElementValue(e, "description");
                itemcategory = getElementValue(e, "category");
                itemguid = getElementValue(e, "guid");
                itempubdate = getElementValue(e, "pubDate");
                output.append("document.write('<tr><td>&nbsp;<font class=\"topContentSectionLink\">&#149;</font> <a href=\"" + itemlink + "\"><font class=\"topContentSectionLink\"><u><b>" + itemtitle + "</b></u></font></a></td></tr>')\n");

                if (showDesc.equals("yes")) { 
                    output.append("document.write('<tr><td><font class=contentTextTiny>" + itemdescription + "</font></td></tr>')\n");	
                }
            }
            output.append("document.write('</table>')\n");
            output.append("document.close()");
        }
        return output.toString();
    }

    /**
    * Returns a String that contains the value associated with a specific
    * node or element in an XML document.
    *
    * @param    el      the root element to use when looking for the value
    *           name    the name of the element to look for below the root
    * @return   a       string containing the value of the XML element
    */
    private String getElementValue(Element el, String name) {	
        String retval = "";
        try {
            NodeList nlist=el.getElementsByTagName(name);
            retval = escapeStr(nlist.item(0).getFirstChild().getNodeValue());
        } catch (Exception e) {

        }
        return retval;
    }
}

%><%

FeedServer myFeed = new FeedServer();
String output = myFeed.generateFeed(request);
response.getWriter().print(output);

%>